/*
 * \file: Table.h
 */

#ifndef TABLE_H
#define TABLE_H

#include <iostream>
#include <vector>
#include <map>

enum color_type { white, black, red };
class Disc;

class Table {
    bool vege;
    std::vector<std::vector<Disc*> > t;  ///< itt tároljuk a korongokat
    static const int MaxDir = 8;         ///< ennyi irányunk van
    struct dir_type { int dx, dy; } dirs[MaxDir];  ///< irányokat tároló tömb

    /// adott X pozíció érvényes?
    /// @param x - X pozíció
    /// @return true, ha érvényes
    bool validx(size_t x) const { return x < t.size(); }

    /// adott Y pozíció érvényes?
    /// @param y - Y pozíció
    /// @return true, ha érvényes
    bool validy(size_t y) const { return y < t[0].size(); }
public:
    class Position;  ///< belső osztály a pozíció tárolására, absztrakció szintjén hasonlít az iterátorra
    typedef std::map<int, Disc*> neighbors_type;  ///< szomszédokat megadó típus (irány, pointer)

    /// Tabla konstruktor
    /// @param x - X méret
    /// @param y - Y méret
    Table(int nx = 8, int ny = 8);

    /// Szomszédok lekérdezése.
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @return szomszédok map-je (irány, Disc*)
    neighbors_type getNeighbors(int x, int y);

    /// Adott irányban levő szomszéd lekérdezése
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @param dir - irány [0..MaxDir]
    /// @return szomszéd pointere, vagy null
    Disc* getNext(int x, int y, int dir) const;

    /// Új pozíció létrehozása
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @return pozíció
    Position newPosition(int x, int y) { return Position(x, y, this); }

    /// Adott hely tesztelése
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @return true, ha szabad a hely
    bool probe(int x, int y);

    /// Korong felrakása a táblára
    /// @param d - korong pointere
    void setDisc(Disc* d);

    bool jatekVege() const { return vege; }

    /// Tabla kirajzolása
    void list() const;
    ~Table();
  /// Belső osztály.
  /// A korongok számára nyújt általánosított pozíció információt
  class Position {
    friend class Table; ///< Table osztály hozzáférhet a privát adatokhoz
    int x, y;           ///< koordináták
    Table *tp;          ///< Erre a táblára hivatkozok.
    int getx() const { return x; }
    int gety() const { return y; }
  public:
    Position(int x, int y, Table* tp) :x(x), y(y), tp(tp) {}

    /// Szomszédok lekérdezése.
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @return szomszédok map-je (irány, Disc*)
    neighbors_type getNeighbors() { return tp->getNeighbors(x, y); }

    /// Adott irányban levő szomszéd lekérdezése
    /// @param x - X pozíció
    /// @param y - Y pozíció
    /// @param dir - irány [0..MaxDir]
    /// @return szomszéd pointere, vagy null
    Disc* getNext(int dir) { return tp->getNext(x, y, dir); }
  };
};

#endif // TABLE_H
