/**
 * \file labirint.cpp
 *    Labirint objektum a labirintus programhoz.
 * $Id: labirint.cpp 499 2015-05-05 10:53:36Z szebi $
 */

#include <iostream>
#include "memtrace.h"
#include "labirint.h"

using std::cout;
using std::endl;

/*
 * Labirint konstruktora.
 * Létrehozza a cellákat és mátrix-szerűen kitölti.
 */
Labirint::Labirint(int i, int j) :n(i), m(j) {
    const Cella::irany F = Cella::F;            // így rövidebben tudunk rá hivatkozni
    const Cella::irany J = Cella::J;            // a baj az, hogy az enum nem objektum
    const Cella::irany L = Cella::L;            // így a using nem megy
    const Cella::irany B = Cella::B;
    lp = new Cella[n*m];                        // terület lefoglalása
    for (i = 0; i < n; i++)                     // feltöltés
      for (j = 0; j < m; j++) {
        Cella *p = &lp[I(i, j)];                // ez az aktuális cella
        p->SetPos(i, j);                        // geometriai szomszédok
        p->SetObj(&URES);                       // feltöltés az üres objektummal
        if (i) p->SetSz(F, &lp[I(i-1, j)]);     // Felső szomszéd
        if (i < n-1) p->SetSz(L, &lp[I(i+1, j)]);// Alsó szomszéd
        if (j) p->SetSz(B, &lp[I(i, j-1)]);     // Bal szomszéd
        if (j < m-1) p->SetSz(J, &lp[I(i, j+1)]);// Jobb szomszéd
      }
}

/*
 * Labirint Lep tagfüggvénye
 * Meghívja az objektumok Lep tagfüggvényét.
 * Kezeli, ha az adott ciklusban már léptetett objektum lépne újra
 */
void Labirint::Lep() {
    Obj *p, *lo = NULL;         // munkaváltozók
    Obj **ls = new Obj*[m];     // előző sor tárolója
    int i, j;

    for (j = 0; j < m; j++)
      ls[j] = NULL;             // első sornál ez üres

    try {
        for (i = 0; i < n; i++)  {
          for (j = 0; j < m; j++) {
            Cella &c = lp[I(i, j)];
            if ((p = c.GetObj())        // Lépteti, ha nem nullpointer
                    && lo != p          // és, ha nem ez volt az előző
                    && ls[j] != p) {    // és, ha nem az előző sorból lépett ide
              ls[j] = lo = p;
              p->Lep(c);
            }
          }
        }
    } catch (...) {             // Hiba ág: Lép dobhat kivételt, ezért a
        delete[] ls;            // lefoglalt területet fel kell szabadítani
        throw;                  // és továbbdobjuk
    }
    // normál ág
    delete[] ls;                // lefoglalt területet fel kell szabadítani
}

/*
 * Labirint Print tagfüggvénye.
 */
void Labirint::Print() {
    for (int i = 0; i < n; i++)  {
      for (int j = 0; j < m; j++) {
        Obj *p = lp[I(i, j)].GetObj();          // pointer
        cout << p->Jele();                      // Obj jele
      }
      cout << endl;
    }
}
