/**
 * \file labirint.h
 *    Labirint objektum deklarációja a labirintus programhoz.
 * $Id: labirint.h 499 2015-05-05 10:53:36Z szebi $
 */

#ifndef LABIRINT_H
#define LABIRINT_H
#include "cella.h"

/**
 * Labirint objektum.
 * A konstruktor mátrix alakban helyezi el a cellákat.
 * A különleges kapcsolatokat tagfüggvényekkel kell kialakítani.
 */
class Labirint {
    int n, m;                                   ///< labirintus mérete
    Cella *lp;                                  ///< cellák tárolója
    Labirint(const Labirint&);                  // hogy ne lehessen használni
    Labirint& operator=(const Labirint&);       // hogy ne lehessen használni

    /// Indexelés segédfüggvénye.
    /// labirintus mérete alapján kiszámítja az indexkifejezést
    /// @param i - sor
    /// @param j - oszlop
    /// @return megfelelő cellapobjektum indexe
    int I(int i, int j) { return (i * m + j); } // indexkifejezés
public:
    /// Konstruktor.
    /// Létrehozza a cellákat és matrix-szerüen kitölti.
    /// @param i - sorok száma
    /// @param j - oszlopok száma
    Labirint(int i, int j);                     // konstruktor

    /// Objektum elhelyezése.
    /// Beirja az adott objektum cimét a megadott cellára
    /// @param i - sor
    /// @param j - oszlop
    /// @param a - referencia az objektumra
    void SetObj(int i, int j, Obj &a) { lp[I(i, j)].SetObj(&a); }// Obj beállítása

    /// Cella objektumának lekérdezése.
    /// Kiolvassa az adot cella objektumpointerét
    /// @param i - sor
    /// @param j - oszlop
    /// @return objektum cime
    Obj *GetObj(int i, int j) { return(lp[I(i, j)].GetObj()); } // Obj lekérdezése

    /// Cella szomszédságának beálltása.
    /// @param i - sor
    /// @param j - oszlop
    /// @param n - irány (J, L, B, F)
    /// @param szi - somszéd sorindexe
    /// @param szj - somszéd oszlopindexe
    void SetSz(int i, int j, int n, int szi, int szj) { // szomszéd beállítása
        lp[I(i, j)].SetSz(n, &lp[I(szi, szj)]); }

    /// Cella szomszédságának lekérdezése
    /// @param i - sor
    /// @param j - oszlop
    /// @param n  - irány (J, L, B, F)
    /// @return szomszédcellára mutató pointer
    Cella *GetSz(int i, int j, int n) { return (lp[I(i, j)].GetSz(n)); } // szomszéd lekérdezése

    /// Labirintus összes objektumának léptetése.
    void Lep();                                 // lépteti az objektumokat

    /// Labirintus "kirajzolása"
    void Print();                               // kirajzolja a pillanatnyi állást

    ~Labirint() { delete[] lp; }                // destruktor
};
#endif
